{*******************************************************}
{                                                       }
{       TFormHider                                      }
{                                                       }
{*******************************************************}

unit FormHider;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, ComCtrls, Menus, Clipbrd;

type
  TFormHider = class(TCustomControl)
  private
    fActive: Boolean;
    fFormIsHidden: Boolean;
    fKeyRegistered: Boolean;
    fHotKey: TShortCut;
    fOnBeforeAction: TNotifyEvent;
    fOnAfterAction: TNotifyEvent;
    procedure SetActive(Value: Boolean);
    procedure UpdatefHotKey(Value: TShortCut);
  protected
    procedure loaded; override;
    procedure WndProc(var Message: TMessage); override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    procedure   HideOrUnhideForm;
  published
    property Active: Boolean read fActive write SetActive;
    property KeyRegistered: Boolean read fKeyRegistered;
    property FormIsHidden: Boolean read fFormIsHidden write fFormIsHidden;
    property HotKey: TShortCut read fHotKey write UpdatefHotKey;
    property OnBeforeAction: TNotifyEvent read fOnBeforeAction write fOnBeforeAction;
    property OnAfterAction: TNotifyEvent read fOnAfterAction write fOnAfterAction;
  end;

procedure Register;

implementation

var Old: TShortCut;

constructor TFormHider.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  fKeyRegistered := False;
  fActive := False;
  fFormIsHidden := False;
  fHotKey := TextToShortCut('');
  Visible := false;
  height := 28;
  width := 28;
end;

procedure TFormHider.Paint;
var
   PaintRect: TRect;
   Bmp: TBitmap;
   ResName : array[0..12] of char;
begin
   if (csDesigning in ComponentState) then
   begin
        PaintRect := Rect(0, 0, Width, Height);
        Frame3D(Canvas, PaintRect, clBtnHighlight, clBtnShadow, 1);
        Bmp := TBitmap.create;
        StrPCopy(ResName,classname);
        Bmp.handle := LoadBitmap(hinstance,ResName);
        BitBlt(canvas.handle, 2, 2, ClientRect.Right-4, ClientRect.Bottom-4, Bmp.canvas.handle, 0, 0, SRCCOPY);
        Bmp.free;
   end;
end;

destructor TFormHider.Destroy;
begin
  inherited Destroy;
end;

procedure TFormHider.SetActive(Value: Boolean);
begin
     fActive := Value;
end;

procedure TFormHider.UpdatefHotKey(Value: TShortCut);
begin
  if not (csDesigning in ComponentState) then
   begin
    if fKeyRegistered then
     if (not UnregisterHotKey(handle,IDHOT_SNAPWINDOW)) and (ShortCutToText(fHotKey)<>'') then
      begin
       fKeyRegistered := False;
       Application.MessageBox(PChar('Unable to unregister hide/unhide hotkey "'+ShortCutToText(fHotKey)+'".'+#13#13+
       'Another application may have locked it.  Try closing all other applications.'),'Hotkey Error',MB_OK+MB_ICONERROR);
      end
     else
      begin
       Old := fHotKey;
       fHotKey := Value;
       Loaded;
       { Key was not registered }
       if not fKeyRegistered then
        Loaded;
      end
    else
     if ShortCutToText(Value)<>'' then
      begin
       fHotKey := Value;
       Loaded;
      end
   end;
end;

procedure TFormHider.loaded;
var Shift: TShiftState;
    Key: Word;
    fsModifiers: Cardinal;
begin
  if not (csDesigning in ComponentState) then
  if (ShortCutToText(fHotKey)<>'') then
  begin
   ShortCutToKey(fHotKey,Key,Shift);
   fsModifiers := Cardinal(nil);
   if ssCtrl in Shift then
    fsModifiers := fsModifiers + MOD_CONTROL;
   if ssAlt in Shift then
    fsModifiers := fsModifiers + MOD_ALT;
   if ssShift in Shift then
    fsModifiers := fsModifiers + MOD_SHIFT;
   fKeyRegistered := True;
   if (not RegisterHotkey(handle, IDHOT_SNAPWINDOW, fsModifiers{MOD_ALT}, Key{VK_F12})) then
    begin
      fKeyRegistered := False;
      Application.MessageBox(PChar('Unable to register hide/unhide hotkey "'+ShortCutToText(fHotKey)+'".  It may be locked '+
      'by another application.'+#13#13+
      'Try changing the hotkey or closing all other applications.'),'Hotkey Error',MB_OK+MB_ICONERROR);
      fHotKey := Old;
    end;
  end;
end;

procedure TFormHider.WndProc(var Message: TMessage);
begin
    if (Message.Msg = WM_HOTKEY) and (Message.wParam = IDHOT_SNAPWINDOW) and fActive then
     HideOrUnhideForm;
    inherited WndProc(Message);
end;

procedure TFormHider.HideOrUnhideForm;
begin
  if (assigned(fOnBeforeAction)) then fOnBeforeAction(self);
  fFormIsHidden := not fFormIsHidden;
  if fFormIsHidden then
   begin
    ShowWindow(Application.MainForm.handle,SW_HIDE);
    ShowWindow(Application.handle,SW_HIDE);
   end
  else
   begin
    ShowWindow(Application.handle,SW_SHOW);
    ShowWindow(Application.MainForm.handle,SW_SHOW);
    Application.Restore;
    Application.BringToFront;
   end;
  if (assigned(fOnAfterAction)) then fOnAfterAction(self);
end;

procedure Register;
begin
  RegisterComponents('My Components', [TFormHider]);
end;

end.
